@@ -23,11 +23,9 @@ import posixpath
 from gettext import gettext as _
 from string import Template
 
-from checkbox.lib.iterator import NEXT
-
-from checkbox.job import Job, UNINITIATED
+from checkbox.job import UNINITIATED
 from checkbox.user_interface import (UserInterface,
-    YES_ANSWER, NO_ANSWER, SKIP_ANSWER,
+    NEXT, YES_ANSWER, NO_ANSWER, SKIP_ANSWER,
     ANSWER_TO_STATUS, STATUS_TO_ANSWER)
 
 # Import to register HyperTextView type with gtk
@@ -191,8 +189,13 @@ class GTKInterface(UserInterface):
 
     def _run_dialog(self, dialog=None):
         def on_dialog_response(dialog, response, self):
-            self.direction = response
-            gtk.main_quit()
+            # Keep dialog alive when the button that has been clicked
+            # isn't meant to move to a previous/next window
+            # To do this, please link the RESPONSE_REJECT code to such
+            # button in the glade file
+            if response != gtk.RESPONSE_REJECT:
+                self.direction = response
+                gtk.main_quit()
 
         dialog = dialog or self._dialog
         dialog.connect("response", on_dialog_response, self)
@@ -260,23 +263,40 @@ class GTKInterface(UserInterface):
     def show_check(self, text, options=[], default=[]):
         # Set buttons
         self._notebook.set_current_page(1)
+        self._set_show("button_select_all", True)
+        self._set_show("button_deselect_all", True)
 
         # Set options
         option_table = {}
-        vbox = self._get_widget("vbox_options")
+        vbox = self._get_widget("vbox_options_list")
         for option in options:
-            label = "_%s" % option.capitalize()
-            radio_button = gtk.CheckButton(label)
-            radio_button.show()
-            option_table[option] = radio_button
-            vbox.pack_start(radio_button, False, False, 0)
+            label = "_%s%s" % (option[0].upper(), option[1:])
+            check_button = gtk.CheckButton(label)
+            check_button.get_child().set_line_wrap(True)
+            check_button.show()
+            option_table[option] = check_button
+            vbox.pack_start(check_button, False, False, 0)
             if option in default:
-                radio_button.set_active(True)
+                check_button.set_active(True)
 
         self._set_hyper_text_view("hyper_text_view_options", text)
 
+        # Set callbacks
+        def click_button(widget, active):
+            for check_button in option_table.itervalues():
+                check_button.set_active(active)
+
+        for button_name in "button_select_all", "button_deselect_all":
+            button = self._get_widget(button_name)
+            button.connect("clicked", click_button,
+                "deselect" not in button_name)
+
         self._run_dialog()
 
+        # Unset buttons
+        self._set_show("button_select_all", False)
+        self._set_show("button_deselect_all", False)
+
         # Get options
         results = []
         for option, check_button in option_table.items():
@@ -294,10 +314,11 @@ class GTKInterface(UserInterface):
         # Set options
         option_group = None
         option_table = {}
-        vbox = self._get_widget("vbox_options")
+        vbox = self._get_widget("vbox_options_list")
         for option in options:
-            label = "_%s" % option.capitalize()
+            label = "_%s%s" % (option[0].upper(), option[1:])
             radio_button = gtk.RadioButton(option_group, label)
+            radio_button.get_child().set_line_wrap(True)
             radio_button.show()
             option_table[option] = radio_button
             vbox.pack_start(radio_button, False, False, 0)
@@ -319,13 +340,113 @@ class GTKInterface(UserInterface):
 
         return result
 
-    def _run_test(self, test):
+    @GTKHack
+    def show_tree(self, text, options={}, default={}):
+        (COLUMN_TEXT, COLUMN_ACTIVE) = range(2)
+
+        # Set buttons
+        self._notebook.set_current_page(1)
+        self._set_show("button_select_all", True)
+        self._set_show("button_deselect_all", True)
+
+        # Set options
+        treestore = gtk.TreeStore(str, bool)
+        treeview = gtk.TreeView(treestore)
+        treeview.set_headers_visible(False)
+        treeview.show()
+
+        def set_options(options, default, parent=None):
+            keys = sorted(options.keys())
+            values = [options[k] for k in keys]
+            for text, child in zip(keys, values):
+                active = text in default
+                iter = treestore.append(parent, [text, active])
+                set_options(child, default.get(text, {}), iter)
+
+        set_options(options, default)
+
+        vbox = self._get_widget("vbox_options_list")
+        vbox.pack_start(treeview, False, False, 0)
+
+        def toggle_tree(iter, active=None):
+            # Set value
+            if active is None:
+                active = not treestore.get_value(iter, COLUMN_ACTIVE)
+            treestore.set_value(iter, COLUMN_ACTIVE, active)
+
+            # Set parents
+            def set_parents(iter):
+                parent = treestore.iter_parent(iter)
+                if parent:
+                    treestore.set_value(parent, COLUMN_ACTIVE, active)
+                    set_parents(parent)
+
+            if active:
+                set_parents(iter)
+
+            # Set children
+            def set_children(iter):
+                child = treestore.iter_children(iter)
+                while child:
+                    treestore.set_value(child, COLUMN_ACTIVE, active)
+                    set_children(child)
+                    child = treestore.iter_next(child)
+
+            set_children(iter)
+
+        # Toggle column
+        cell = gtk.CellRendererToggle()
+        cell.connect("toggled",
+            lambda t, p: toggle_tree(treestore.get_iter_from_string(p)))
+        col = gtk.TreeViewColumn(None, cell, active=COLUMN_ACTIVE)
+        treeview.append_column(col)
+
+        # Text column
+        cell = gtk.CellRendererText()
+        col = gtk.TreeViewColumn(None, cell, text=COLUMN_TEXT)
+        treeview.append_column(col)
+
+        # Select and deselect buttons
+        def click_button(widget, active):
+            iter = treestore.get_iter_first()
+            while iter:
+                toggle_tree(iter, active)
+                iter = treestore.iter_next(iter)
+
+        for button_name in "button_select_all", "button_deselect_all":
+            button = self._get_widget(button_name)
+            button.connect("clicked", click_button,
+                "deselect" not in button_name)
+
+        self._run_dialog()
+
+        # Unset buttons
+        self._set_show("button_select_all", False)
+        self._set_show("button_deselect_all", False)
+
+        # Get options
+        def get_results(iter):
+            results = {}
+            while iter:
+                active = treestore.get_value(iter, COLUMN_ACTIVE)
+                if active:
+                    text = treestore.get_value(iter, COLUMN_TEXT)
+                    children = treestore.iter_children(iter)
+                    results[text] = get_results(children)
+
+                iter = treestore.iter_next(iter)
+
+            return results
+
+        results = get_results(treestore.get_iter_first())
+        vbox.remove(treeview)
+
+        return results
+
+    def _run_test(self, test, runner):
         self._set_sensitive("button_test", False)
 
-        message = _("Running test %s...") % test["name"]
-        job = Job(test["command"], test.get("environ"),
-            test.get("timeout"), test.get("user"))
-        (status, data, duration) = self.show_progress(message, job.execute)
+        (status, data, duration) = runner(test)
 
         description = Template(test["description"]).substitute({
             "output": data.strip()})
@@ -336,13 +457,13 @@ class GTKInterface(UserInterface):
         self._set_sensitive("button_test", True)
 
     @GTKHack
-    def show_test(self, test):
+    def show_test(self, test, runner):
         self._set_sensitive("button_test", False)
         self._notebook.set_current_page(2)
 
         # Set test description
         if re.search(r"\$output", test["description"]):
-            self._run_test(test)
+            self._run_test(test, runner)
         else:
             self._set_hyper_text_view("hyper_text_view_test",
                 test["description"])
@@ -354,7 +475,7 @@ class GTKInterface(UserInterface):
             if self._handler_id:
                 button_test.disconnect(self._handler_id)
             self._handler_id = button_test.connect("clicked",
-                lambda w, t=test: self._run_test(t))
+                lambda w, t=test, r=runner: self._run_test(t, r))
 
         self._set_text_view("text_view_comment", test.get("data", ""))
         self._set_active(STATUS_TO_BUTTON[test.get("status", UNINITIATED)])
