@@ -41,9 +41,11 @@ class LockPrompt(Plugin):
 
     def register(self, manager):
         super(LockPrompt, self).register(manager)
+
         self._lock = None
 
-        self._manager.reactor.call_on("prompt-begin", self.prompt_begin)
+        self._manager.reactor.call_on(
+            "prompt-begin", self.prompt_begin, -1000)
 
     def prompt_begin(self, interface):
         directory = posixpath.dirname(self.filename)
