@@ -19,12 +19,10 @@
 import logging
 
 from checkbox.lib.dmi import Dmi
-from checkbox.lib.file import write
 from checkbox.lib.safe import safe_md5sum
 
-from checkbox.properties import Path, String
+from checkbox.properties import String
 from checkbox.plugin import Plugin
-from checkbox.registry import registry_eval_recursive
 
 
 class SystemInfo(Plugin):
@@ -32,42 +30,44 @@ class SystemInfo(Plugin):
     # System ID to exchange information with the server.
     system_id = String(required=False)
 
-    # Filename where System ID is cached
-    filename = Path(default="%(checkbox_data)s/system")
-
     def register(self, manager):
         super(SystemInfo, self).register(manager)
 
+        self.resource = None
+
+        for (rt, rh) in [
+             ("begin-persist", self.begin_persist),
+             ("report-device", self.report_device)]:
+            self._manager.reactor.call_on(rt, rh)
+
         # System report should be generated early.
         self._manager.reactor.call_on("report", self.report, -10)
 
-        self._manager.reactor.call_on("gather-persist", self.gather_persist)
-
-    def gather_persist(self, persist):
+    def begin_persist(self, persist):
         self.persist = persist.root_at("system_info")
 
+    def report_device(self, resources):
+        for resource in resources:
+           if resource.get("category") == "CHASSIS":
+               self.resource = resource
+
     # TODO: report this upon gathering
     def report(self):
         system_id = self.system_id or self.persist.get("system_id")
         if not system_id:
-            devices = registry_eval_recursive(self._manager.registry,
-                "device.category == 'CHASSIS'")
-            if not devices:
+            resource = self.resource
+            if resource is None or "product" not in resource:
                 return
 
-            device = devices[0]
-            if not device:
-                return
-
-            chassis_type = Dmi.chassis_name_to_type[device.product]
+            chassis_type = Dmi.chassis_name_to_type[resource["product"]]
 
             fingerprint = safe_md5sum()
             for field in [
                     "Computer",
                     "unknown",
                     chassis_type,
-                    device.vendor,
-                    device.model]:
+                    resource.get("vendor", ""),
+                    resource.get("model", "")]:
                 fingerprint.update(str(field))
 
             system_id = fingerprint.hexdigest()
@@ -75,7 +75,6 @@ class SystemInfo(Plugin):
 
         message = system_id
         logging.info("System ID: %s", message)
-        write(self.filename, message)
         self._manager.reactor.fire("report-system_id", message)
 
 
